package com.cecilia.common.base;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.concurrent.atomic.AtomicLong;

public class WebRequestSequence {

    private static final Logger LOGGER = LoggerFactory.getLogger(WebRequestSequence.class);
    private static final String IGNORED_TRACE = "Ignored_Trace";

    private static ThreadLocal<WebRequestSequence> s_sequences = new ThreadLocal<>();

    private static final AtomicLong s_nextId = new AtomicLong(System.currentTimeMillis());

    private static final boolean TRACE_ENABLED = initTrace();

    private static String prefix = "";

    private String requestSequence = "";
    private static String traceIdDefault = "";

    static {
        InetAddress addr;
        try {
            addr = InetAddress.getLocalHost();
        } catch (UnknownHostException var1) {
            addr = InetAddress.getLoopbackAddress();
        }
        if (addr != null) {
            byte[] address = addr.getAddress();
            int lastByte = ((int) address[address.length - 1] & 0xff);
            prefix = lastByte + "-";
        }
    }

    private static boolean initTrace() {
        boolean traceAgent = false;
        try {
            Class.forName("org.apache.skywalking.apm.agent.SkyWalkingAgent");
            traceAgent = true;
        } catch (ClassNotFoundException e) {
            LOGGER.info("Trace agent org.apache.skywalking.apm.agent.SkyWalkingAgent not found.");
        }

        boolean traceContext = false;
        try {
            Class.forName("org.apache.skywalking.apm.toolkit.trace.TraceContext");
            traceContext = true;
        } catch (ClassNotFoundException e) {
            LOGGER.info("Trace context org.apache.skywalking.apm.toolkit.trace.TraceContext not found.");
        }

        if (traceAgent && traceContext) {
            LOGGER.info("Distribute traceId enabled.");
            return true;
        }
        LOGGER.info("Local request sequence enabled.");
        return false;
    }

    private WebRequestSequence() {
        //
    }

    public static WebRequestSequence current() {
        WebRequestSequence seq = s_sequences.get();
        if (seq == null) {
            seq = new WebRequestSequence();
            s_sequences.set(seq);
        }
        return seq;
    }

    public String getRequestSequence() {
        return this.requestSequence;
    }

    public void setupRequestSequence() {
        if (TRACE_ENABLED) {
            String traceId = traceIdDefault;
            if (!IGNORED_TRACE.equals(traceId)) {
                this.requestSequence = traceId;
                return;
            }
        }

        long id = s_nextId.addAndGet(1);
        this.requestSequence = prefix + String.valueOf(id);
    }

    public void clearRequestSequence() {
        s_sequences.remove();
    }
}